#include "Arduino.h"
#include "programmer.h"
#include "program.h"
#include "max80_gpio.h"
#include "USB.h"
extern USBCDC USBSerial; 
uint8_t cmd = 0;
int32_t length = 0;

int passthrough = 0;
int lastled = 0;

void ReadCmdLength(uint8_t *cmd, int32_t *length) {
  while(Serial.available() < 5);

  *cmd = Serial.read();
  *length = 0;
  uint8_t *dataLengthbytes = (uint8_t*)length;

  // for (int i = 0; i < 4; i++) {
  for (int i = 3; i >= 0; i--) {
    dataLengthbytes[i] = Serial.read();
  }
}

String ReadParam(int32_t length) {
  String param = "";
  for (int32_t i = 0; i < length; i++) {
    param += (char)Serial.read();
  }
  return param;
}

void ProgrammerInit() {
    set_pins(MAX80_FPGA_TDI , MAX80_FPGA_TDO , MAX80_FPGA_TCK , MAX80_FPGA_TMS, MAX80_LED3);

    pinMode(MAX80_FPGA_TDI, OUTPUT);
    pinMode(MAX80_FPGA_TDO, INPUT_PULLUP);
    pinMode(MAX80_FPGA_TCK, OUTPUT);
    pinMode(MAX80_FPGA_TMS, OUTPUT);
}

String tmpParam;

void ProgrammerLoop() {
  if (passthrough) {
    // Serial passthrough
    if(Serial.available()) {
      USBSerial.write(Serial.read());
      digitalWrite( MAX80_LED3, (lastled++)&1);
    }

    if(USBSerial.available()) {
      Serial.write(USBSerial.read());
      digitalWrite( MAX80_LED3, (lastled++)&1);
    }

    return;
  }

  // Programming Mode
  if (Serial.available() >= 5) {
    ReadCmdLength(&cmd, &length);
    switch (cmd) {
      case CMD_START_SVF:
        jtag_program(DATA_TYPE_SVF, MODE_SERIAL);
        break;
      case CMD_START_XSVF:
        jtag_program(DATA_TYPE_XSVF, MODE_SERIAL);
        break;
      case CMD_QUERY:
        Serial.print("[QUERY] Chip ID: ");
        printf("%08x\n", jtag_chip_id());
        break;
      case CMD_STOP:
        Serial.println("[RDY]");
        break;
      case CMD_PASSTHROUGH:
        Serial.println("[PASS] Switching to Serial Passthrough");
        Serial.print("[PASS] Serial2 baudrate set to ");
        Serial.println(length);
        passthrough = 1;
        pinMode( MAX80_LED3, OUTPUT);
        digitalWrite( MAX80_LED3, LOW);
        Serial.println("[PASS] READY");
        return;
      default:
        Serial.println("[CTRL] ERROR");
    }
  }
}

int fetch_next_block(uint8_t *buffer, int length) {

  Serial.print("[CTRL] R");
  Serial.println(length);

  uint8_t cmd;
  int32_t dataLength;
  ReadCmdLength(&cmd, &dataLength);
  if (dataLength <= 0) {
    return -1;
  }

  int readBytes = 0;

  while (readBytes < dataLength) {
    while(!Serial.available());
    buffer[readBytes] = Serial.read();
    readBytes++;
  }
  Serial.println("[CTRL] A");

  return readBytes;
}
